
CubesTypesBar = {}

local BORDER_LEFT = 20+100+20
local BACKGROUND_COLOR = Color.new(0,0,0,170)
local BACKGROUND_COLOR_SIDE_BAR = Color.new(0,0,0,120)

local buttons_insight = {}

local scroll_insight_bar = 0

local selected_item = 1

local mouse_pressed_in = false
local mouse_last_pos = Int2.new(-1000,-1000)

local MAX_HEIGHT = Misc.getRunningEnvironment() == 'ENVIRONMENT_WINDOWS_STORE_WITH_ADS' and (Misc.getWindowSize():height()-250) or Misc.getWindowSize():height()

--------------------------------------------------------

local function getMinScroll_() return 0 end
local function getMaxScroll_() return 150.0*#buttons_insight-MAX_HEIGHT+150 end

local function fixScrollOverLimits_()
	if scroll_insight_bar > -1.0*getMinScroll_() then scroll_insight_bar = -1.0*getMinScroll_() end
	if scroll_insight_bar < -1.0*getMaxScroll_() then scroll_insight_bar = -1.0*getMaxScroll_() end
end

local function drawSideBar_(x)
	local local_size_scroll = MAX_HEIGHT * MAX_HEIGHT / (getMaxScroll_()+MAX_HEIGHT)
	local pos_y = (-1.0*scroll_insight_bar - getMinScroll_()) / (getMaxScroll_() - getMinScroll_()) * MAX_HEIGHT
	pos_y = pos_y*(MAX_HEIGHT-local_size_scroll)/MAX_HEIGHT
	Scene2D.drawRectangle(Int2.new(x,pos_y), Int2.new(x+20, pos_y+local_size_scroll), BACKGROUND_COLOR_SIDE_BAR)
end

--------------------------------------------------------

function CubesTypesBar.init()

	buttons_insight[1] = Button.new('data/removal_mode.png', '')
	buttons_insight[1]:scale(Number2.new(100/128,100/128))
	
	local i = 0
	for i = 0, CubesEngine.get():getNbCubeTypes()-1 do
		local fn1, fn2, fn3, fn4, fn5, fn6, is_collidable = CubesEngine.get():getCubeTypeInfos(i)
		buttons_insight[i+2] = Button.new(fn4, '', Int2.new(0, 0), Int2.new(128, 128))
	end
	
	for key, button in pairs(buttons_insight) do
		button:colorMouseOver(Color.new(0,0,0,128))
	end
end

--------------------------------------------------------

function CubesTypesBar.drawPanel(cubes_engine)

	MAX_HEIGHT = Misc.getRunningEnvironment() == 'ENVIRONMENT_WINDOWS_STORE_WITH_ADS' and (Misc.getWindowSize():height()-250) or Misc.getWindowSize():height()

	Scene2D.drawRectangle(Int2.new(0,0), Int2.new(BORDER_LEFT, MAX_HEIGHT), BACKGROUND_COLOR)
	drawSideBar_(BORDER_LEFT)
	
	local i = 0
	for key, button in pairs(buttons_insight) do
		button:scale(Number2.new(100/button:width()*button:scale():x(),100/button:height()*button:scale():y()))
		button:draw(Int2.new(20, i*150+50 + scroll_insight_bar))
		if selected_item==i-1 then
			if selected_item==-1 then
				Scene2D.drawRectangle(button:position(), button:position()+button:size(), Color.new(128,128,128,255), 5, false)
			else
				Scene2D.drawRectangle(button:position(), button:position()+button:size(), Color.new(255,255,255,255), 5, false)
			end
		end
		i=i+1
	end	
end

--------------------------------------------------------

function CubesTypesBar.onMousePressed(mouse_button, mouse_pos, is_double_click)

	mouse_pressed_in = mouse_pos:x() < BORDER_LEFT
	
	if mouse_pressed_in then
		mouse_last_pos = mouse_pos:clone()
		
		if buttons_insight[1]:isIn(mouse_pos) then
			selected_item = -1
		else
			local i = 0
			for key, button in pairs(buttons_insight) do
				if button:isIn(mouse_pos) then
					selected_item = i-1
				end
				i=i+1
			end	
		end
		return true
	end
	return false
end

--------------------------------------------------------

function CubesTypesBar.onMouseMoved(is_mouse_pressed, mouse_pos)
	for key, button in pairs(buttons_insight) do
		button:checkIsOver(mouse_pos)
	end
	
	if is_mouse_pressed and mouse_pressed_in then
		scroll_insight_bar = scroll_insight_bar - (mouse_last_pos:y() - mouse_pos:y())
		fixScrollOverLimits_()
		mouse_last_pos = mouse_pos:clone()
		return true
	end
	
	return false
end

--------------------------------------------------------

function CubesTypesBar.onMouseWheelChanged(wheel_delta, mouse_pos)
	if mouse_pos:x() < BORDER_LEFT then
		scroll_insight_bar = scroll_insight_bar + wheel_delta/2
		fixScrollOverLimits_()
		return true
	end	
	return false
end

--------------------------------------------------------

function CubesTypesBar.drop()

	for key, button in pairs(buttons_insight) do
		button:drop()
	end	

end

--------------------------------------------------------

function CubesTypesBar.getSelectedCubeType()
	return selected_item
end

--------------------------------------------------------
