
EditingPanel = {}

local BORDER_RIGHT = 20+100+20
local COLOR_BORDER_ICON = Color.new(255,255,255,255)
local BACKGROUND_COLOR = Color.new(0,0,0,170)
local BACKGROUND_COLOR_SIDE_BAR = Color.new(0,0,0,120)

local buttons = {}

local selected_editing_type = 0
local editing_type_changed = false

local function getMinScroll_() return -750 end

local scroll_insight_bar = -1.0*getMinScroll_()

local mouse_pressed_in = false
local mouse_last_pos = Int2.new(-1000,-1000)

--------------------------------------------------------


local function getMaxScroll_() return 2000-Misc.getWindowSize():height()+300 end --150.0*#buttons_insight-Misc.getWindowSize():height()+150 end

local function fixScrollOverLimits_()
	if scroll_insight_bar > -1.0*getMinScroll_() then scroll_insight_bar = -1.0*getMinScroll_() end
	if scroll_insight_bar < -1.0*getMaxScroll_() then scroll_insight_bar = -1.0*getMaxScroll_() end
end

local function drawSideBar_(x)
	local local_size_scroll = Misc.getWindowSize():height() * Misc.getWindowSize():height() / (getMaxScroll_()+Misc.getWindowSize():height())
	local pos_y = (-1.0*scroll_insight_bar - getMinScroll_()) / (getMaxScroll_() - getMinScroll_()) * Misc.getWindowSize():height()
	pos_y = pos_y*(Misc.getWindowSize():height()-local_size_scroll)/Misc.getWindowSize():height()
	Scene2D.drawRectangle(Int2.new(x,pos_y), Int2.new(x+20, pos_y+local_size_scroll), BACKGROUND_COLOR_SIDE_BAR)
end

--------------------------------------------------------

--local function fixScrollOverLimits_()
--	if scroll_insight_bar > MIN_SCROLL then scroll_insight_bar = MIN_SCROLL end
--	local max_h = -2000+Misc.getWindowSize():height()-300
--	if scroll_insight_bar < max_h then scroll_insight_bar = max_h end
--end

--------------------------------------------------------

function EditingPanel.init()

	buttons['loadsave_menu'] = Button.new('data/editing_panel/save.png', '')
	buttons['options_menu'] = Button.new('data/editing_panel/gear.png', '')
	buttons['center_camera'] = Button.new('data/editing_panel/camera.png', '')
	buttons['dot'] = Button.new('data/editing_panel/dot.png', '')
	buttons['cuboid'] = Button.new('data/editing_panel/cuboid.png', '')
	buttons['sphere'] = Button.new('data/editing_panel/sphere.png', '')
	buttons['cut'] = Button.new('data/editing_panel/cut.png', '')
	buttons['copy'] = Button.new('data/editing_panel/copy.png', '')
	buttons['paste'] = Button.new('data/editing_panel/paste.png', '')
	buttons['paste_stretched'] = Button.new('data/editing_panel/paste_stretched.png', '')
	buttons['random_floating_islands'] = Button.new('data/editing_panel/random_floating_islands.png', '')
	buttons['random_floating_islands_dense'] = Button.new('data/editing_panel/random_floating_islands_dense.png', '')
	buttons['active_object'] = Button.new('data/editing_panel/star.png', '')
	buttons['set_position_player'] = Button.new('data/editing_panel/set_position_player.png', '')
	
	for key, button in pairs(buttons) do
		button:colorMouseOver(Color.new(0,0,0,128))
	end
end

--------------------------------------------------------

function EditingPanel.draw(buffer_size_or_nil)
	local left_pos = Misc.getWindowSize():width()-BORDER_RIGHT
	Scene2D.drawRectangle(Int2.new(left_pos,0), Misc.getWindowSize(), BACKGROUND_COLOR)
	drawSideBar_(left_pos-20)
	
	buttons['options_menu']:draw(Int2.new(left_pos+20, -750 + 50 + scroll_insight_bar))
	buttons['options_menu']:setBorder(COLOR_BORDER_ICON, selected_editing_type==-3 and 5 or 0)
	
	buttons['loadsave_menu']:draw(Int2.new(left_pos+20, -600 + 50 + scroll_insight_bar))
	buttons['loadsave_menu']:setBorder(COLOR_BORDER_ICON, selected_editing_type==-2 and 5 or 0)
	
	buttons['center_camera']:draw(Int2.new(left_pos+20, -300 + 50 + scroll_insight_bar))
	buttons['center_camera']:setBorder(COLOR_BORDER_ICON, selected_editing_type==-1 and 5 or 0)
	
	buttons['dot']:draw(Int2.new(left_pos+20, 50 + scroll_insight_bar))
	buttons['dot']:setBorder(COLOR_BORDER_ICON, selected_editing_type==0 and 5 or 0)
	
	buttons['cuboid']:draw(Int2.new(left_pos+20, 200 + scroll_insight_bar))
	buttons['cuboid']:setBorder(COLOR_BORDER_ICON, selected_editing_type==1 and 5 or 0)
	
	buttons['sphere']:draw(Int2.new(left_pos+20, 350 + scroll_insight_bar))
	buttons['sphere']:setBorder(COLOR_BORDER_ICON, selected_editing_type==2 and 5 or 0)
	
	if buffer_size_or_nil ~= nil then
		Scene2D.drawText(
			'['..buffer_size_or_nil:x()..','..buffer_size_or_nil:y()..','..buffer_size_or_nil:z()..']',
			Int2.new(left_pos+30, 600 + scroll_insight_bar),
			18,
			Color.new(255,255,255,255)
			)
	end
	
	buttons['cut']:draw(Int2.new(left_pos+20, 650 + scroll_insight_bar))
	buttons['cut']:setBorder(COLOR_BORDER_ICON, selected_editing_type==3 and 5 or 0)
	
	buttons['copy']:draw(Int2.new(left_pos+20, 800 + scroll_insight_bar))
	buttons['copy']:setBorder(COLOR_BORDER_ICON, selected_editing_type==4 and 5 or 0)
	
	buttons['paste']:draw(Int2.new(left_pos+20, 950 + scroll_insight_bar), 
		buffer_size_or_nil ~= nil and 1 or 0.5)
	buttons['paste']:setBorder(COLOR_BORDER_ICON, selected_editing_type==5 and 5 or 0)
	
	buttons['paste_stretched']:draw(Int2.new(left_pos+20, 1100 + scroll_insight_bar), 
		buffer_size_or_nil ~= nil and 1 or 0.5)
	buttons['paste_stretched']:setBorder(COLOR_BORDER_ICON, selected_editing_type==6 and 5 or 0)
	
	buttons['random_floating_islands']:draw(Int2.new(left_pos+20, 1400 + scroll_insight_bar))
	buttons['random_floating_islands']:setBorder(COLOR_BORDER_ICON, selected_editing_type==7 and 5 or 0)
	
	buttons['random_floating_islands_dense']:draw(Int2.new(left_pos+20, 1550 + scroll_insight_bar))
	buttons['random_floating_islands_dense']:setBorder(COLOR_BORDER_ICON, selected_editing_type==8 and 5 or 0)
	
	buttons['active_object']:draw(Int2.new(left_pos+20, 1850 + scroll_insight_bar))
	buttons['active_object']:setBorder(COLOR_BORDER_ICON, selected_editing_type==9 and 5 or 0)
	
	buttons['set_position_player']:draw(Int2.new(left_pos+20, 2150 + scroll_insight_bar))
	buttons['set_position_player']:setBorder(COLOR_BORDER_ICON, selected_editing_type==10 and 5 or 0)
end

--------------------------------------------------------

function EditingPanel.onMousePressed(mouse_button, mouse_pos, is_double_click, buffer_size_or_nil)

	mouse_pressed_in = mouse_pos:x() > Misc.getWindowSize():width()-BORDER_RIGHT

	if mouse_pressed_in then
		mouse_last_pos = mouse_pos:clone()
		
		if     buttons['options_menu']:isIn(mouse_pos) then
			if selected_editing_type == -3 then selected_editing_type = -1000 else selected_editing_type = -3 end
		elseif buttons['loadsave_menu']:isIn(mouse_pos) then
			if selected_editing_type == -2 then selected_editing_type = -1000 else selected_editing_type = -2 end
		elseif buttons['center_camera']:isIn(mouse_pos) then
			selected_editing_type = -1
		elseif buttons['dot']:isIn(mouse_pos) then
			selected_editing_type = 0
		elseif buttons['cuboid']:isIn(mouse_pos) then
			selected_editing_type = 1
		elseif buttons['sphere']:isIn(mouse_pos) then
			selected_editing_type = 2
		elseif buttons['cut']:isIn(mouse_pos) then
			selected_editing_type = 3
		elseif buttons['copy']:isIn(mouse_pos) then
			selected_editing_type = 4
		elseif buttons['paste']:isIn(mouse_pos) and buffer_size_or_nil ~= nil then
			selected_editing_type = 5
		elseif buttons['paste_stretched']:isIn(mouse_pos) and buffer_size_or_nil ~= nil then
			selected_editing_type = 6
		elseif buttons['random_floating_islands']:isIn(mouse_pos) then
			selected_editing_type = 7
		elseif buttons['random_floating_islands_dense']:isIn(mouse_pos) then
			selected_editing_type = 8
		elseif buttons['active_object']:isIn(mouse_pos) then
			selected_editing_type = 9
		elseif buttons['set_position_player']:isIn(mouse_pos) then
			selected_editing_type = 10
		end
		editing_type_changed = true
		return true
	end
	return false
end

--------------------------------------------------------

function EditingPanel.getSelectedEditingType()
	return selected_editing_type
end

function EditingPanel.acknowledgeEditingTypeChange()
	if editing_type_changed then
		editing_type_changed = false 
		return true
	end
	return false
end

--------------------------------------------------------

function EditingPanel.onMouseMoved(is_mouse_pressed, mouse_pos)
	for key, button in pairs(buttons) do
		button:checkIsOver(mouse_pos)
	end
	
	if is_mouse_pressed and mouse_pressed_in then
		scroll_insight_bar = scroll_insight_bar - (mouse_last_pos:y() - mouse_pos:y())
		fixScrollOverLimits_()
		mouse_last_pos = mouse_pos:clone()
		return true
	end
	
	return false
end

--------------------------------------------------------

function EditingPanel.onMouseWheelChanged(wheel_delta, mouse_pos)
	if mouse_pos:x() > Misc.getWindowSize():width()-BORDER_RIGHT then
		scroll_insight_bar = scroll_insight_bar + wheel_delta/2
		fixScrollOverLimits_()
		return true
	end	
	return false
end

--------------------------------------------------------

function EditingPanel.drop()
	for key, button in pairs(buttons) do
		button:drop()
	end
end

--------------------------------------------------------
