
dofile "data/CubesTypesBar.lua"
dofile "data/ActiveObjectsBar.lua"
dofile "data/EditingPanel.lua"
dofile "data/OptionsMenu.lua"
dofile "data/LoadSaveMenu.lua"

--------------------------------------------------------

local TIME_MOVE_CAMERA_CENTER = 500.0

--------------------------------------------------------

local camera_distance = 100.0
local camera_angle_x = 30
local camera_angle_y = -30

--------------------------------------------------------

local creation_step = 0
local cuboid_limit = nil
local is_ok_creating = true
local cuboid_creation_zone = nil
local cuboid_creation_zone_step1 = nil
local cuboid_initial_position_player = nil
local creation_pos2d = nil
local remaining_axis = nil

-- 3d intersection, picking and mode picking/mouse
local pos_3d_intersection = Number3.new(0,0,0)
local button_switch_picking_mode = nil
local mode_picking = true

local sphere_creation_sphere = nil

local prev_mouse_pos = Int2.new(-1000,-1000)

local cuboid_camera_center = nil
local prev_camera_center = nil
local actual_camera_center = nil
local time_end_tmp_pos_center_mvt = nil

local cubes_engine = nil

local buffer_cubes = nil

local cubes_modified_during_game = {}
local cuboid_removed_cube = nil


local sound_creative_click = nil
local sound_cheerful_click = nil

local X_BUTTONS = 20+100+20+40+120*4

local cuboid_plane_camera = nil

--------------------------------------------------------

Editor = {}

-------------------------------------------------------------------------------------------------------------------------
-- local functions
-------------------------------------------------------------------------------------------------------------------------

local function getRealPosFromPickingPos(pos3d, distance_correction)
	local SIZE_ONE_CUBE = cubes_engine:getSizeOneCube()
	local pos_in_cube = pos3d - Maths.getNormalizedVector(Scene3D.cameraPosition(), pos3d) * distance_correction
	local new_x = math.floor((pos_in_cube:x())/SIZE_ONE_CUBE)*SIZE_ONE_CUBE +5.0
	local new_y = math.floor((pos_in_cube:y())/SIZE_ONE_CUBE)*SIZE_ONE_CUBE +5.0
	local new_z = math.floor((pos_in_cube:z())/SIZE_ONE_CUBE)*SIZE_ONE_CUBE +5.0
	return Number3.new(new_x,new_y,new_z)
end

local function getRealPosFromPickingPos2(pos3d, distance_correction)
	local SIZE_ONE_CUBE = cubes_engine:getSizeOneCube()
	local pos_in_cube = pos3d - Maths.getNormalizedVector(Scene3D.cameraPosition(), pos3d) * distance_correction
	local new_x = math.floor((pos_in_cube:x()+5)/SIZE_ONE_CUBE)*SIZE_ONE_CUBE --+5.0
	local new_y = math.floor((pos_in_cube:y()+5)/SIZE_ONE_CUBE)*SIZE_ONE_CUBE --+5.0
	local new_z = math.floor((pos_in_cube:z()+5)/SIZE_ONE_CUBE)*SIZE_ONE_CUBE --+5.0
	return Number3.new(new_x,new_y,new_z)
end

local function getCubesPosFromRealPosPicking(pos)
	return Int3.new((pos:x()-5) / cubes_engine:getSizeOneCube(),(pos:y()-5) / cubes_engine:getSizeOneCube(),(pos:z()-5) / cubes_engine:getSizeOneCube())
end

local function displaysCubesTypesPanel()
	return (EditingPanel.getSelectedEditingType() >= 0 and EditingPanel.getSelectedEditingType() < 3) 
		or EditingPanel.getSelectedEditingType() == 7 or EditingPanel.getSelectedEditingType() == 8
end

local function displaysActiveObjectsPanel()
	return EditingPanel.getSelectedEditingType() == 9
end

local function fixCameraOutOfBounds_()
	while camera_angle_x < 0    do camera_angle_x=camera_angle_x+360 end
	while camera_angle_x > 360  do camera_angle_x=camera_angle_x-360 end
	while camera_angle_y < -180 do camera_angle_y=camera_angle_y+360 end
	while camera_angle_y > 180  do camera_angle_y=camera_angle_y-360 end
	if camera_distance < 1 then 
		camera_distance = 1
	elseif camera_distance > Scene3D.fogDistance()*0.8 then 
		camera_distance = Scene3D.fogDistance()*0.8
	end
end

local function mvtSel(x,y,z)
	return CubesEngine.get():fromCubePosToRealPos(getCubesPosFromRealPos(Number3.new(x,y,z)))
end

local function cuboidCreationPosition(x,y,z)
	local pt = cuboid_creation_zone_step1 + Number3.new(x,y,z)
	cuboid_creation_zone:size(
		Number3.new(math.abs(pt:x()), math.abs(pt:y()), math.abs(pt:z()))) 
	local mod_product = Number3.new((pt:x() > 0 and 1 or -1), (pt:y() > 0 and 1 or -1), (pt:z() > 0 and 1 or -1))
	cuboid_creation_zone:position(cuboid_limit:position() + Number3.new(
		cuboid_creation_zone:size():x()*mod_product:x()*0.5, 
		cuboid_creation_zone:size():y()*mod_product:y()*0.5, 
		cuboid_creation_zone:size():z()*mod_product:z()*0.5))-- + mod2
end

local function manageCreationStepRotate(mouse_pos)

	local diff = creation_pos2d - mouse_pos

	local cam = camera_angle_x+45
	local camY = camera_angle_y
	
	local a1 = (cam >= 360 or cam < 90)
	local a2 = (cam >= 90 and cam < 180)
	local a3 = (cam >= 180 and cam < 270)
	local a4 = (cam >= 270 and cam < 360)
	
	local b1 = (camY < -45)
	local b2 = (camY >= -45 and camY < 45)
	local b3 = (camY >= 45)
	
	local ns1 = nil
	local ns2 = nil
	
	local x_remains = true
	local y_remains = true
	local z_remains = true
	
	if b1 then
		if     a1 then ns1=mvtSel(0,0,-diff:y()) ; z_remains=false
		elseif a2 then ns1=mvtSel(-diff:y(),0,0) ; x_remains=false
		elseif a3 then ns1=mvtSel(0,0,diff:y()) ; z_remains=false
		elseif a4 then ns1=mvtSel(diff:y(),0,0) ; x_remains=false
		end
	elseif b2 then 
		ns1=mvtSel(0,diff:y(),0) ; y_remains=false
	elseif b3 then
		if     a1 then ns1=mvtSel(0,0,diff:y()) ; z_remains=false
		elseif a2 then ns1=mvtSel(diff:y(),0,0) ; x_remains=false
		elseif a3 then ns1=mvtSel(0,0,-diff:y()) ; z_remains=false
		elseif a4 then ns1=mvtSel(-diff:y(),0,0) ; x_remains=false
		end
	end
	
	if     a1 then ns2=mvtSel(-diff:x(),0,0) ; x_remains=false
	elseif a2 then ns2=mvtSel(0,0,diff:x()) ; z_remains=false
	elseif a3 then ns2=mvtSel(diff:x(),0,0) ; x_remains=false
	elseif a4 then ns2=mvtSel(0,0,-diff:x()) ; z_remains=false
	end
		
	if     x_remains then remaining_axis = 'x'
	elseif y_remains then remaining_axis = 'y'
	elseif z_remains then remaining_axis = 'z'
	end
	
	cuboid_creation_zone_step1 = ns1+ns2
	cuboidCreationPosition(0, 0, 0)
	
end

-------------------------------------------------------------------------------------------------------------------------
-- Editor functions
-------------------------------------------------------------------------------------------------------------------------

function Editor.init()
	CubesTypesBar.init()
	ActiveObjectsBar.init()
	EditingPanel.init()
	OptionsMenu.init()
	LoadSaveMenu.init()
	
	cuboid_camera_center = Cuboid3D.new('data/editor/camera_icon.png')
	cuboid_camera_center:size(Number3.new(10, 10, 10))
	--cuboid_camera_center:position(Number3.new(100, 100, 100))
	cuboid_camera_center:position(Number3.new(0, 0, 0))
	
	cuboid_limit = Cuboid3D.new('data/editor/red_square_small.png')
	cuboid_limit:size(Number3.new(5, 5, 5))
	
	cuboid_creation_zone = Cuboid3D.new('data/editor/red_square_large.png')
	cuboid_creation_zone:size(Number3.new(0, 0, 0))
	
	sphere_creation_sphere = Sphere3D.new('data/editor/selection_sphere.png')
	
	cubes_engine = CubesEngine.get()
	
	cuboid_initial_position_player = Cuboid3D.new('data/editor/initial_position_player.png')
	cuboid_initial_position_player:size(Number3.new(10,10,10))
	cuboid_initial_position_player:position(Number3.new(0,30,0))
			
	cuboid_removed_cube = Cuboid3D.new('data/removed_cube.png')
	cuboid_removed_cube:size(Number3.new(CubesEngine.get():getSizeOneCube(), CubesEngine.get():getSizeOneCube(), CubesEngine.get():getSizeOneCube()))
	cuboid_removed_cube:enableLighting(false)
	
	sound_creative_click = Sound.new('data/sound/106123__j1987__clickselect__mod.wav')
	sound_cheerful_click = Sound.new('data/sound/243020__plasterbrain__game-start.wav')
	
	button_switch_picking_mode = Button.new('data/editor/switch_picking_mode.png', '')
	button_switch_picking_mode:colorMouseOver(Color.new(0,0,0,128))
	
	cuboid_plane_camera = Cuboid3D.new('data/editor/plane_camera.png')
end

function Editor.draw2D()

	OptionsMenu.draw()
	LoadSaveMenu.draw()

	if displaysCubesTypesPanel() then
		CubesTypesBar.drawPanel(cubes_engine)
	end
	if displaysActiveObjectsPanel() then
		ActiveObjectsBar.drawPanel()
	end
	
	EditingPanel.draw(buffer_cubes~=nil and buffer_cubes:size() or nil)
	
	button_switch_picking_mode:draw(
		Int2.new(X_BUTTONS,Misc.getWindowSize():height()-button_switch_picking_mode:height()-40))

	--Scene2D.drawText(''..pos_3d_intersection:x()..'     -     '..pos_3d_intersection:y()..'     -     '..pos_3d_intersection:z(), Int2.new(10, 10))
	--local cube_pos_inside  = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, 4.0))
	--local cube_pos_outside = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, -4.0))
	--Scene2D.drawText(''..cube_pos_inside:x()..'     -     '..cube_pos_inside:y()..'     -     '..cube_pos_inside:z(), Int2.new(10, 30))
	--Scene2D.drawText(''..cube_pos_outside:x()..'     -     '..cube_pos_outside:y()..'     -     '..cube_pos_outside:z(), Int2.new(10, 50))

end

function Editor.manage()

	actual_camera_center = cuboid_camera_center:position()
	
	if time_end_tmp_pos_center_mvt ~= nil and Misc.getTime() < time_end_tmp_pos_center_mvt then
		local diff = (time_end_tmp_pos_center_mvt - Misc.getTime())/TIME_MOVE_CAMERA_CENTER
		actual_camera_center:x( cuboid_camera_center:position():x()*(1-diff)+prev_camera_center:x()*diff )
		actual_camera_center:y( cuboid_camera_center:position():y()*(1-diff)+prev_camera_center:y()*diff )
		actual_camera_center:z( cuboid_camera_center:position():z()*(1-diff)+prev_camera_center:z()*diff )
	end
	
	Scene3D.setCameraLookAt(actual_camera_center, camera_angle_x, camera_angle_y, camera_distance)
end

function Editor.draw3D()
	
	local cuboid_plane_camera_is_visible = false
	local cuboid_creation_zone_is_visible = false
	local cuboid_limit_is_visible = false
	local sphere_creation_sphere_is_visible = false
	local cuboid_removed_cube_is_visible = false

	if mode_picking then
		if Misc.getRunningEnvironment() == "ENVIRONMENT_WINDOWS_STORE" or Misc.getRunningEnvironment() == "ENVIRONMENT_WINDOWS_DESKTOP_DIRECTX" or Misc.getRunningEnvironment() == "ENVIRONMENT_WINDOWS_STORE_WITH_ADS" then
			pos_3d_intersection = CubesEngine.get():get3DPosFrom2D(Input.getMousePosition())
		else
			pos_3d_intersection = Scene3D.get3DPosFrom2D(Input.getMousePosition())
		end
	else
		local x_shift = (Input.getMousePosition():x()-Misc.getWindowSize():x()*0.5)*-camera_distance/600.0
		local y_shift = (Input.getMousePosition():y()-Misc.getWindowSize():y()*0.5)*-camera_distance/600.0
		
		local pos_shift = nil
		
		local cam = camera_angle_x+45
		local camY = camera_angle_y
		
		local a1 = (cam >= 360 or cam < 90)
		local a2 = (cam >= 90 and cam < 180)
		local a3 = (cam >= 180 and cam < 270)
		local a4 = (cam >= 270 and cam < 360)
		
		local b1 = (camY < -70)
		local b2 = (camY >= -70 and camY < 40)
		local b3 = (camY >= 40)
		
		-- vertical shift
		if     b1 then
			if     a1 then pos_shift = Number3.new(0,0,-y_shift)
			elseif a2 then pos_shift = Number3.new(-y_shift,0,0)
			elseif a3 then pos_shift = Number3.new(0,0,y_shift)
			elseif a4 then pos_shift = Number3.new(y_shift,0,0)
			end
		elseif b2 then 
			pos_shift = Number3.new(0,y_shift,0)
		elseif b3 then
			if     a1 then pos_shift = Number3.new(0,0,y_shift)
			elseif a2 then pos_shift = Number3.new(y_shift,0,0)
			elseif a3 then pos_shift = Number3.new(0,0,-y_shift)
			elseif a4 then pos_shift = Number3.new(-y_shift,0,0)
			end
		end
		
		-- horizontal shift
		if     a1 then pos_shift = pos_shift + Number3.new(-x_shift,0,0)
		elseif a2 then pos_shift = pos_shift + Number3.new(0,0,x_shift)
		elseif a3 then pos_shift = pos_shift + Number3.new(x_shift,0,0)
		elseif a4 then pos_shift = pos_shift + Number3.new(0,0,-x_shift)
		end
		
		-- final position
		pos_3d_intersection = actual_camera_center + pos_shift
		
		if creation_step == 0 then
			-- cuboid_plane_camera size and position
			cuboid_plane_camera:size((actual_camera_center - pos_3d_intersection) * 4.0)
			cuboid_plane_camera:position(actual_camera_center)
			--cuboid_plane_camera:draw()
			cuboid_plane_camera_is_visible = true
		end
	end
	
	local selected_editing_type = EditingPanel.getSelectedEditingType()
	
	if selected_editing_type==0 then
		if is_ok_creating then
			--cuboid_creation_zone:draw()
			cuboid_creation_zone_is_visible = true
		end
	elseif selected_editing_type==5 then
		if is_ok_creating and buffer_cubes then
			--cuboid_limit:draw()
			cuboid_limit_is_visible = true
			local tmp_size = cuboid_creation_zone:size()
			cuboid_creation_zone:size(CubesEngine.get():fromCubePosToRealPos(buffer_cubes:size())-Number3.new(5,5,5))
			cuboid_creation_zone:position(cuboid_limit:position() + cuboid_creation_zone:size()*0.5)
			--cuboid_creation_zone:draw()
			cuboid_creation_zone_is_visible = true
			cuboid_creation_zone:size(tmp_size)
		end
	elseif selected_editing_type>0 then
	
		if (is_ok_creating or creation_step > 0) and (selected_editing_type~=6 or buffer_cubes) then
			--cuboid_limit:draw()
			cuboid_limit_is_visible = true
			if creation_step > 0 then
				local sinv = 2.5+5.0*(math.sin(Misc.getTime()*0.005)*0.5+0.5)
				if selected_editing_type==2 then
					cuboid_limit:size(Number3.new(sinv, sinv, sinv))
				elseif creation_step == 1 then
					if remaining_axis == 'x' then
						cuboid_limit:size(Number3.new(2, sinv, sinv))
					elseif remaining_axis == 'y' then
						cuboid_limit:size(Number3.new(sinv, 2, sinv))
					elseif remaining_axis == 'z' then
						cuboid_limit:size(Number3.new(sinv, sinv, 2))
					end
				elseif creation_step == 2 or creation_step == 3 then
					if remaining_axis == 'x' then
						cuboid_limit:size(Number3.new(sinv, 2, 2))
					elseif remaining_axis == 'y' then
						cuboid_limit:size(Number3.new(2, sinv, 2))
					elseif remaining_axis == 'z' then
						cuboid_limit:size(Number3.new(2, 2, sinv))
					end
				end
				
				if selected_editing_type==2 then
					--sphere_creation_sphere:draw()
					sphere_creation_sphere_is_visible = true
				else
					local tmp_size = cuboid_creation_zone:size()
					cuboid_creation_zone:size(cuboid_creation_zone:size()+Number3.new(2,2,2))
					--cuboid_creation_zone:draw()
					cuboid_creation_zone_is_visible = true
					cuboid_creation_zone:size(tmp_size)
				end
			end
		end
	end
	
	if creation_step == 0 then
		cuboid_limit:size(Number3.new(5, 5, 5))
	end
	--cuboid_initial_position_player:draw()

	for key,cube_info in pairs(cubes_modified_during_game) do
		if cube_info.type_ == 'REMOVAL' then
			cuboid_removed_cube:position(CubesEngine.get():fromCubePosToRealPos(cube_info.pos))
			--cuboid_removed_cube:draw()
			cuboid_removed_cube_is_visible = true
		end
	end

	cuboid_camera_center:draw()
	if cuboid_plane_camera_is_visible then cuboid_plane_camera:draw() end
	if cuboid_creation_zone_is_visible then cuboid_creation_zone:draw() end
	if cuboid_limit_is_visible then cuboid_limit:draw() end
	if sphere_creation_sphere_is_visible then sphere_creation_sphere:draw() end
	if cuboid_removed_cube_is_visible then cuboid_removed_cube:draw() end
end

--------------------------------------------------------

function Editor.closeAllMenus()
	LoadSaveMenu.close()
	OptionsMenu.close()
end

--------------------------------------------------------

function Editor.onMousePressed(mouse_button, mouse_pos, is_double_click)

	local is_pressed_options = OptionsMenu.onMousePressed(mouse_button, mouse_pos, is_double_click, CubesEngine.get())
	local is_pressed_loadsave = LoadSaveMenu.onMousePressed(mouse_button, mouse_pos, is_double_click)
	
	if is_pressed_options then LoadSaveMenu.close() end
	if is_pressed_loadsave then OptionsMenu.close() end
	
	if is_pressed_options or is_pressed_loadsave then return true end
	
	local is_pressed = false
	if displaysCubesTypesPanel() then 
		is_pressed = is_pressed or CubesTypesBar.onMousePressed(mouse_button, mouse_pos, is_double_click) 
	end
	if displaysActiveObjectsPanel() then 
		is_pressed = is_pressed or ActiveObjectsBar.onMousePressed(mouse_button, mouse_pos, is_double_click) 
	end
	is_pressed = is_pressed or EditingPanel.onMousePressed(mouse_button, mouse_pos, is_double_click, 
		buffer_cubes~=nil and buffer_cubes:size() or nil)

	local selected_editing_type = EditingPanel.getSelectedEditingType()
	if EditingPanel.acknowledgeEditingTypeChange() then
		creation_step = 0
		
		if selected_editing_type==-3 then
			OptionsMenu.openOrClose()
			LoadSaveMenu.close()
		elseif selected_editing_type==-2 then
			Editor.resetCubesRemovedDuringGame()
			LoadSaveMenu.openOrClose()
			OptionsMenu.close()
		else
			OptionsMenu.close()
			LoadSaveMenu.close()
		end
	end
	
	if not is_pressed then
		if button_switch_picking_mode:isIn(mouse_pos) then
			mode_picking = not mode_picking
			creation_step = 0
			is_pressed = true
		end
	end

	if not is_pressed then
	
		--if mouse_button == 'LEFT_BUTTON' then
		local cube_pos_inside  = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, 4.0))
		local cube_pos_outside = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, -4.0))
		
		if selected_editing_type==-1 or (mouse_button == 'RIGHT_BUTTON' and is_double_click) then
			if creation_step > 0 then creation_step = 0 end
		end
		
		if (cubes_engine:getCubeTypeAt(cube_pos_inside) ~= -1 or not mode_picking)  -- click hits a cube
			and (selected_editing_type==-1 or (mouse_button == 'RIGHT_BUTTON' and is_double_click)) then
			prev_camera_center = actual_camera_center:clone()
			cuboid_camera_center:position(pos_3d_intersection)
			time_end_tmp_pos_center_mvt = Misc.getTime() + TIME_MOVE_CAMERA_CENTER
			
		elseif is_ok_creating and mouse_button ~= 'RIGHT_BUTTON' then
		
			if selected_editing_type==0 then 
				sound_creative_click:play()
				if CubesTypesBar.getSelectedCubeType() == -1 then -- deletion of a cube
					cubes_engine:makeCube(getCubesPosFromRealPos(cuboid_creation_zone:position()), -1)
				else -- add of a cube
					cubes_engine:makeCube(getCubesPosFromRealPos(cuboid_creation_zone:position()), CubesTypesBar.getSelectedCubeType())
				end
			
			elseif selected_editing_type==1 or selected_editing_type==3 
				or selected_editing_type==4 or selected_editing_type==6 then
				
				if creation_step == 0 then
					cuboid_creation_zone:size(Number3.new(0,0,0))
					creation_step = 1
					creation_pos2d = mouse_pos:clone()
					cuboid_creation_zone_step1 = Number3.new(0,0,0)
					
				elseif creation_step == 2 then
					creation_step = 3
				else
					creation_step = 0
				end
				
			elseif selected_editing_type==2 then
				if creation_step == 0 then
					cuboid_creation_zone:size(Number3.new(0,0,0))
					sphere_creation_sphere:radius(0)
					sphere_creation_sphere:position(cuboid_limit:position())
					creation_step = 1
					creation_pos2d = mouse_pos:clone()
					cuboid_creation_zone_step1 = Number3.new(0,0,0)
				end
				
			elseif selected_editing_type==5 then
				sound_creative_click:play()
				if buffer_cubes ~= nil then 
					buffer_cubes:paste(getCubesPosFromRealPos(cuboid_limit:position()))
				end
				
			elseif selected_editing_type==7 then
				sound_cheerful_click:play()
				cubes_engine:makeRandomFloatingIslands(getCubesPosFromRealPos(cuboid_limit:position()), CubesTypesBar.getSelectedCubeType(), 40, 3, false)
			
			elseif selected_editing_type==8 then
				sound_cheerful_click:play()
				cubes_engine:makeRandomFloatingIslands(getCubesPosFromRealPos(cuboid_limit:position()), CubesTypesBar.getSelectedCubeType(), 40, 3, true)
			
			elseif selected_editing_type==9 then
				sound_creative_click:play()
				local selected_active_object = ActiveObjectsBar.getSelectedActiveObject()
				if selected_active_object == '' then
					ActiveObjects.get():removeSphere(cuboid_limit:position(), cubes_engine:getSizeOneCube()*0.5)
				else
					ActiveObjects.get():removeSphere(cuboid_limit:position(), cubes_engine:getSizeOneCube()*0.5)
					ActiveObjects.get():add(selected_active_object, cuboid_limit:position())
				end
				
			elseif selected_editing_type==10 then
				sound_creative_click:play()
				cuboid_initial_position_player:position(cuboid_limit:position())
			end
		end
	end
	
	return is_pressed
end

function Editor.onMouseMoved(is_mouse_pressed, mouse_pos)

	OptionsMenu.onMouseMoved(is_mouse_pressed, mouse_pos)
	LoadSaveMenu.onMouseMoved(is_mouse_pressed, mouse_pos)	

	if displaysCubesTypesPanel() and CubesTypesBar.onMouseMoved(is_mouse_pressed, mouse_pos) then return end
	if displaysActiveObjectsPanel() and ActiveObjectsBar.onMouseMoved(is_mouse_pressed, mouse_pos) then return end
	if EditingPanel.onMouseMoved(is_mouse_pressed, mouse_pos) then return end
	
	local cube_pos_inside  = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, 4.0))
	local cube_pos_outside = getCubesPosFromRealPosPicking(getRealPosFromPickingPos(pos_3d_intersection, -4.0))
	
	local selected_editing_type = EditingPanel.getSelectedEditingType()
	
	if not mode_picking or cubes_engine:getCubeTypeAt(cube_pos_inside) ~= -1 or creation_step > 0 then
		-- click hits a cube
		if creation_step == 0 then
			cuboid_limit:position(getRealPosFromPickingPos2(pos_3d_intersection, -4.0))
		end
		is_ok_creating = true
	else
		is_ok_creating = false
	end
	
	if selected_editing_type==0 then
			if CubesTypesBar.getSelectedCubeType() == -1 then -- deletion of a cube
				cuboid_creation_zone:position(getRealPosFromCubesPos(cube_pos_inside)+Number3.new(5,5,5))
			else -- add of a cube
				cuboid_creation_zone:position(getRealPosFromCubesPos(cube_pos_outside)+Number3.new(5,5,5))
			end
			cuboid_creation_zone:size(Number3.new(12,12,12))
			
	elseif selected_editing_type>0 then
		if creation_step ~= 0 then
			if selected_editing_type==2 then -- sphere creation
				local diff = math.floor((creation_pos2d:y() - mouse_pos:y() + .5) / 20
					+ (creation_pos2d:x() - mouse_pos:x() + .5) / 20)
				sphere_creation_sphere:radius(math.abs(diff)*cubes_engine:getSizeOneCube())
			else -- cube creation and other
				if creation_step == 1 then
					manageCreationStepRotate(mouse_pos)
				elseif creation_step == 2 or creation_step == 3 then
					local diff = math.floor(
						(creation_pos2d:y() - mouse_pos:y() + .5) / 10) * cubes_engine:getSizeOneCube()
					if     remaining_axis == 'x' then cuboidCreationPosition(diff, 0, 0)
					elseif remaining_axis == 'y' then cuboidCreationPosition(0, diff, 0)
					elseif remaining_axis == 'z' then cuboidCreationPosition(0, 0, diff)
					end
				end
			end
		end
	
	end

	if creation_step == 0 then
		if is_mouse_pressed and not (prev_mouse_pos:x() == -1000) and not (prev_mouse_pos:y() == -1000) then
			camera_angle_x = camera_angle_x + prev_mouse_pos:x() - mouse_pos:x()
			if false then
				camera_angle_y = camera_angle_y - prev_mouse_pos:y() + mouse_pos:y()
			else
				camera_angle_y = camera_angle_y + prev_mouse_pos:y() - mouse_pos:y()
			end
			fixCameraOutOfBounds_()
		end
		prev_mouse_pos = mouse_pos:clone()
	end
	
	button_switch_picking_mode:checkIsOver(mouse_pos)
end

function Editor.onMouseReleased(mouse_button, mouse_pos)
	local selected_editing_type = EditingPanel.getSelectedEditingType()
	if selected_editing_type == 1 or selected_editing_type == 3
		or selected_editing_type == 4 or selected_editing_type == 6 then -- cube creation or other
		if creation_step == 1 then
			if cuboid_creation_zone:size():equals(Number3.new(0,0,0)) then
				-- nothing will be created, giving up
				creation_step = 0
			else
				creation_step = 2
				creation_pos2d = mouse_pos:clone()
			end
		elseif creation_step == 3 then
			local min_cubes = getCubesPosFromRealPos(
				cuboid_creation_zone:position()-cuboid_creation_zone:size()*0.5+Number3.new(1,1,1))
			local max_cubes = getCubesPosFromRealPos(
				cuboid_creation_zone:position()+cuboid_creation_zone:size()*0.5+Number3.new(1,1,1))

			if selected_editing_type==1 then
				sound_creative_click:play()
				cubes_engine:makeCuboid(min_cubes, max_cubes, CubesTypesBar.getSelectedCubeType())
			elseif selected_editing_type==3 or selected_editing_type==4 then
				if buffer_cubes ~= nil then buffer_cubes:drop() end
				if selected_editing_type == 3 then sound_creative_click:play() end
				buffer_cubes = BufferOfCubesAndActiveObjects.newFromCopyOrCutCuboid(
					min_cubes, max_cubes, selected_editing_type==3)
			elseif selected_editing_type==6 then
				sound_creative_click:play()
				if buffer_cubes ~= nil then buffer_cubes:paste(min_cubes, max_cubes) end
			end
			creation_step = 0
		end
	elseif selected_editing_type==2 then -- sphere creation
		if creation_step == 1 then
			sound_creative_click:play()
			
			creation_step = 0
			cubes_engine:makeSphere(
				getCubesPosFromRealPos(sphere_creation_sphere:position()), 
				sphere_creation_sphere:radius()/cubes_engine:getSizeOneCube(),
				CubesTypesBar.getSelectedCubeType())
		end
	end
end

--------------------------------------------------------

function Editor.onMouseWheelChanged(wheel_delta, mouse_pos)
	local is_pressed = false
	if displaysCubesTypesPanel() then 
		is_pressed = (is_pressed or CubesTypesBar.onMouseWheelChanged(wheel_delta, mouse_pos)) 
	end
	if displaysActiveObjectsPanel() then 
		is_pressed = (is_pressed or ActiveObjectsBar.onMouseWheelChanged(wheel_delta, mouse_pos)) 
	end
	is_pressed = is_pressed or EditingPanel.onMouseWheelChanged(wheel_delta, mouse_pos)
	if not is_pressed then
		local delta_mod = (wheel_delta > 0 and 0.9090909 or 1.1)
		--if (abs(camera_distance * delta_mod - camera_distance)>200 then
		camera_distance = camera_distance * (delta_mod)-- / 120)
	end
	fixCameraOutOfBounds_()
end

function Editor.onSemanticZoom(value)
	camera_distance = camera_distance * value
	fixCameraOutOfBounds_()
end

--------------------------------------------------------

function Editor.getInitialPositionPlayer()
	return cuboid_initial_position_player:position()
end

--------------------------------------------------------

function Editor.drop()

	CubesTypesBar.drop()
	ActiveObjectsBar.drop()
	EditingPanel.drop()
	OptionsMenu.drop()
	LoadSaveMenu.drop()
	
	cuboid_initial_position_player:drop()
	cuboid_camera_center:drop()
	cuboid_limit:drop()
	cuboid_creation_zone:drop()
	sphere_creation_sphere:drop()
	sound_creative_click:drop()
	sound_cheerful_click:drop()
	
	cuboid_removed_cube:drop()
	cuboid_plane_camera:drop()
	button_switch_picking_mode:drop()
end

--------------------------------------------------------

function Editor.removeCubeDuringGame(cube_pos)
	local cube_type = CubesEngine.get():getCubeTypeAt(cube_pos)
	CubesEngine.get():makeCube(cube_pos, -1)
	table.insert(cubes_modified_during_game, { type_='REMOVAL', pos=cube_pos:clone(), id=cube_type })
end

--------------------------------------------------------

function Editor.moveCubeDuringGame(cube_start_pos, cube_end_pos)
	local cube_type = CubesEngine.get():getCubeTypeAt(cube_start_pos)
	CubesEngine.get():moveCube(cube_start_pos, cube_end_pos, 1000)
	table.insert(cubes_modified_during_game, { type_='REMOVAL', pos=cube_start_pos:clone(), id=cube_type })
	table.insert(cubes_modified_during_game, { type_='ADD', pos=cube_end_pos:clone(), id=cube_type })
end

--------------------------------------------------------

--function Editor.addOrReplaceCubeDuringGame(cube_pos, cube_type)
--	local prev_cube_type = CubesEngine.get():getCubeTypeAt(cube_pos)
--	if prev_cube_type ~= -1 then Editor.removeCubeDuringGame(cube_pos) end
--	CubesEngine.get():makeCube(cube_pos, cube_type)
--	table.insert(cubes_modified_during_game, { type_='ADD', pos=cube_pos:clone() })
--end

--------------------------------------------------------

function Editor.resetCubesRemovedDuringGame()
	for i=#cubes_modified_during_game, 1, -1 do
		local cube_info = cubes_modified_during_game[i]
		if cube_info.type_ == 'REMOVAL' then
			CubesEngine.get():makeCube(cube_info.pos, cube_info.id)
		else
			CubesEngine.get():makeCube(cube_info.pos, -1)
		end
	end
	cubes_modified_during_game = {}
end

--------------------------------------------------------

function getCubesPosFromRealPos(pos)
	return Int3.new(math.floor(pos:x() / cubes_engine:getSizeOneCube()),math.floor(pos:y() / cubes_engine:getSizeOneCube()),math.floor(pos:z() / cubes_engine:getSizeOneCube()))
end

function getRealPosFromCubesPos(pos)
	return Number3.new(pos:x() * cubes_engine:getSizeOneCube(),pos:y() * cubes_engine:getSizeOneCube(),pos:z() * cubes_engine:getSizeOneCube())
end
