
--require "string"
--require "math"
--require "table"

dofile "data/strict.lua"
dofile "data/Editor.lua"
dofile "data/ActiveObjectsMgr.lua"
dofile "data/CubesTypesMgr.lua"

----------------------------------------------------------------------------------------------------------------

local ocean = nil
local sound_splash = nil
local is_player_underwater = false
local time_mouse_pressed = nil
local button_switch_editor = nil
local button_reset = nil

local next_hit_lava = 0

local is_editor_mode = true
local is_reset = true

local X_BUTTONS = 20+100+20+40+120*2

local time_next_move_living_grass = -1
local DURATION_NEXT_MOVE_LIVING_GRASS = 1000
local moved_living_grass = {}

local INITIAL_FOG_DISTANCE = 1000

Main = {}

--------------------------------------------------------

function Main.init()

	Scene3D.enableGet3DPosFrom2D(true)
	CubesEngine.get():detailedMode(false, 2)
	
	ocean = Ocean.new("data/ocean/water_top.png", "data/ocean/water_sub.png") -- , 0, 104, 255, 128)
	
	Scene3D.setSkybox(
		"data/skybox/1.png", "data/skybox/2.png", "data/skybox/3.png", 
		"data/skybox/4.png", "data/skybox/5.png", "data/skybox/6.png")

	ActiveObjectsMgr.init()
	CubesTypesMgr.init()
	Editor.init()
	CubesAndActiveObjects.finalizeCubeTypesAndActiveObjectTypes()
	sound_splash = Sound.new("data/sound/splash.wav")
	
	--Player.position(Number3.new(50.0,500.0,50.0))
	Scene3D.fogDistance(INITIAL_FOG_DISTANCE)
	Scene3D.fogColor(Color.new(51, 175, 255, 255))--100, 162, 239, 255)) -- 33afff, not 00b2f0
	
	Scene3D.cameraPosition(Number3.new(0.0,100.0,500.0))
	
	Player.load(true)
	Player.setModel3D(0.05, -1.0, Number3.new(0.0, 12.0, 0.0), 
			"data/obj_female_rogue/female_rogue_static.obj",
			"data/obj_female_rogue/female_rogue_anim1.obj",
			"data/obj_female_rogue/female_rogue_anim2.obj",
			"data/obj_female_rogue/female_rogue_anim3.obj",
			"data/obj_female_rogue/female_rogue_anim4.obj",
			"data/obj_female_rogue/female_rogue_anim5.obj",
			"data/obj_female_rogue/female_rogue_jump.obj")
	-- Player.setModel3D(0.4, -1.0, Number3.new(0.0, 0.0, 0.0), 
			-- "data/obj_player_robot/player_frame1.obj",
			-- "data/obj_player_robot/player_frame1.obj",
			-- "data/obj_player_robot/player_frame2.obj",
			-- "data/obj_player_robot/player_frame3.obj",
			-- "data/obj_player_robot/player_frame4.obj",
			-- "data/obj_player_robot/player_frame5.obj",
			-- "data/obj_player_robot/player_frame1.obj")
	Player.enable(false)
	Player.position(Editor.getInitialPositionPlayer())
	
	button_switch_editor = Button.new('data/main_icons/switch_mode.png', '')
	button_switch_editor:colorMouseOver(Color.new(0,0,0,128))
	
	button_reset = Button.new('data/main_icons/reset.png', '')
	button_reset:colorMouseOver(Color.new(0,0,0,128))
	
	if FileUtil.fileExistsInAppData('last_session.3dworld') then
		CubesAndActiveObjects.loadWorldInAppData('last_session.3dworld')
		Player.position(Editor.getInitialPositionPlayer())
	else
		CubesEngine.get():makeCuboid(Int3.new(-5,-2,-5),Int3.new(5,2,5), 1)
	end
end

--------------------------------------------------------

function resetPlayer()
	Player.setGameOver(false)
	Player.position(Editor.getInitialPositionPlayer())
	Player.speed(Number3.new(0,0,0))
	ActiveObjectsMgr.reset()
	Editor.resetCubesRemovedDuringGame()
end

--------------------------------------------------------

function concatTables(t1,t2) 
	for i=1,#t2 do t1[#t1+1] = t2[i] end
	return t1 
end

function isInArrayInt3(t, pos_to_find)
	for key, pos in pairs(t) do
		if pos:equals(pos_to_find) then return true end
	end
	return false
end

function removeFromArrayInt3(t, pos_to_find)
	for key, pos in pairs(t) do
		if pos:equals(pos_to_find) then table.remove(t, key) ; break ; end
	end
end

function manageInteractiveCubes()
	for i=0,5,5 do
		local posPlayerFront = CubesEngine.get():fromRealPosToCubePos(Player.position() + Number3.new(math.cos(Player.angleY())*10, i, math.sin(Player.angleY())*10))
		if CubesEngine.get():getCubeTypeAt(posPlayerFront) == CubesTypesMgr.getIdLockBlue() and ActiveObjectsMgr.nbKeys() > 0 then
			Editor.removeCubeDuringGame(posPlayerFront)
			ActiveObjectsMgr.nbKeys(ActiveObjectsMgr.nbKeys()-1)
		elseif CubesEngine.get():getCubeTypeAt(posPlayerFront) == CubesTypesMgr.getIdBoxKey() then
			Editor.removeCubeDuringGame(posPlayerFront)
			ActiveObjectsMgr.nbKeys(ActiveObjectsMgr.nbKeys()+1)
		end
	end
	
	local is_colliding_lava = false
	if not Player.isGameOver() then
		for i=0,5,5 do
			local posPlayerFront = CubesEngine.get():fromRealPosToCubePos(Player.position() + Number3.new(math.cos(Player.angleY())*10, i, math.sin(Player.angleY())*10))
			if CubesEngine.get():getCubeTypeAt(posPlayerFront) == CubesTypesMgr.getIdLava() then
				is_colliding_lava = true
			end
		end
		if is_colliding_lava then
			if Misc.getTime() > next_hit_lava then
				ActiveObjectsMgr.loseOneLife()
				next_hit_lava = Misc.getTime() + 1000
			end
		end
	end
	
	manageLivingGrass()
end

function manageLivingGrass()
	
	if Misc.getTime() > time_next_move_living_grass then
		local new_moved_living_grass = {}
		for i=-5,5 do
			for j=-5,5 do
				for k=-5,5 do
					local pos_check = CubesEngine.get():fromRealPosToCubePos(Player.position())+Int3.new(i,j,k)
					if CubesEngine.get():getCubeTypeAt(pos_check) == CubesTypesMgr.getIdLivingGrass() then
						if not isInArrayInt3(moved_living_grass, pos_check) then
							table.insert(new_moved_living_grass, pos_check)
						end
					end
				end
			end
		end
		
		for key, pos in pairs(moved_living_grass) do -- reset moved living grass
			if CubesEngine.get():getCubeTypeAt(pos) == CubesTypesMgr.getIdLivingGrass() then
				Editor.moveCubeDuringGame(pos, pos-Int3.new(1,0,0))
				removeFromArrayInt3(moved_living_grass, pos)
			end
		end
		for key, pos in pairs(new_moved_living_grass) do
			Editor.moveCubeDuringGame(pos, pos+Int3.new(1,0,0))
			table.insert(moved_living_grass, pos+Int3.new(1,0,0))
		end
					
		time_next_move_living_grass = Misc.getTime() + DURATION_NEXT_MOVE_LIVING_GRASS
	end
	
end


function Main.manage()
	if true then
		local playerPos = Player.position()
		if playerPos:y() < -5.0 and not is_player_underwater then
			local playerSpeed = Player.speed()
			local sound_factor = -Player.speed():y() * 3.0 / Misc.getFrameDuration()
			if sound_factor > 200.0 then sound_factor = 200.0
			elseif sound_factor < 0.0 then sound_factor = 0.0
			end
			sound_splash:volume(sound_factor) -- / 256.0
		
			is_player_underwater = true
			sound_splash:play()
			
		elseif playerPos:y() > -5.0 and is_player_underwater then
			is_player_underwater = false
		end
		
		if playerPos:y() < -1500.0 then
			Player.setGameOver(true)
		end
	end

	if not is_editor_mode then
		manageInteractiveCubes()
	end
	
	if is_editor_mode then 
		Editor.manage()
	end

	ocean:manage()
	
	CubesTypesMgr.manage()
	ActiveObjectsMgr.manage(is_editor_mode)
	--other_cubes_engine:manage(0,0,2000)--xp,yp-50.0,zp)
end
	
--------------------------------------------------------

function Main.draw2D()
	
	ocean:draw2D()

	button_switch_editor:draw(Int2.new(X_BUTTONS,Misc.getWindowSize():height()-button_switch_editor:height()-40))
	if not is_reset then
		button_reset:draw(Int2.new(X_BUTTONS+120,Misc.getWindowSize():height()-button_reset:height()-40))
	end
	
	if is_editor_mode then 
		Editor.draw2D()
	else
		ActiveObjectsMgr.draw2D()
	end
end
	
--------------------------------------------------------

function Main.draw3D()
	CubesTypesMgr.manage()
	CubesTypesMgr.draw3D()
	ActiveObjectsMgr.draw3D()
	
	ocean:draw3D()
	
	if is_editor_mode then 
		Editor.draw3D()
	end
end
	
--------------------------------------------------------

function Main.onKeyPressed(key)
end

--------------------------------------------------------

function Main.onMousePressed(mouse_button, mouse_pos)
	
	local is_double_click = false
	if time_mouse_pressed ~= nil and time_mouse_pressed > Misc.getTime() - 300 then is_double_click = true end
	time_mouse_pressed = Misc.getTime()
	
	local is_pressed = false
		
	if button_switch_editor:isIn(mouse_pos) then
		if is_editor_mode then
			Player.enable(true)
			is_editor_mode = false
			is_reset = false
			Editor.closeAllMenus()
		else
			Player.enable(false)
			is_editor_mode = true
		end
		is_pressed = true
	elseif not is_reset and button_reset:isIn(mouse_pos) then
		resetPlayer()
		is_pressed = true
		if is_editor_mode then is_reset = true end
	end
	
	if not is_pressed and is_editor_mode then
		Editor.onMousePressed(mouse_button, mouse_pos, is_double_click)
	end

end

--------------------------------------------------------

function Main.onMouseReleased(mouse_button, mouse_pos)
	Editor.onMouseReleased(mouse_button, mouse_pos)
end

--------------------------------------------------------

function Main.onMouseMoved(is_mouse_pressed, mouse_pos)

	if is_editor_mode then
		Editor.onMouseMoved(is_mouse_pressed, mouse_pos)
	end
	
	button_switch_editor:checkIsOver(mouse_pos)
	button_reset:checkIsOver(mouse_pos)
end

--------------------------------------------------------

function Main.onMouseWheelChanged(wheel_delta, mouse_pos)
	Editor.onMouseWheelChanged(wheel_delta, mouse_pos)
end

--------------------------------------------------------

function Main.onSemanticZoom(value)
	Editor.onSemanticZoom(10.0*value)
end

--------------------------------------------------------

function Main.onSuspending()
	CubesAndActiveObjects.saveWorldInAppData('last_session.3dworld')
end

--------------------------------------------------------

function Main.uninit()
	CubesAndActiveObjects.saveWorldInAppData('last_session.3dworld')
	ActiveObjectsMgr.drop()
	CubesTypesMgr.drop()
	ocean:drop()
	Editor.drop()
	sound_splash:drop()
	button_switch_editor:drop()
	button_reset:drop()
end
